{$ignore-function-results}
{$w no-underscore}
{$setlimit=512 }  { needed to correctly construct set [c1] in procedure CreateKeyLabelArray }

module ncursesPlus;

{ Create a set of key labels that are more user-friendly than those in keyname. }
{ Ncurses uses "keyname" which is really the internal variable name for keys, only }
{ loosely corresponding to the actual text printed on the key. }
{ This module tries to establish "KeyLabels" that are more useful to display to the User, }
{   that is, it attempts to find what is on the key or shown in the terminal emulator. }
{ for example  keyname "KEY_F(1)"  ->  keylabel "F1"   }
{              keyname "KEY_DC"    ->  keylable "del"  }

{ compile for GPC: gp ncursesPlus.pas
               or: gpc --automake -c nursesPlus.pas
}

export ncursesPlus = (keylabelmax, labstrtype, KeyLabels, HasKeySet, TermNamePS );

import ncurses;

const
  keylabelmax = 20;   { length of 20 used for key labels internally to ncurses }
  Shift = 'Shift-';   { Shift prefix; ok to change here to shorter prefix, as "Sh-"  }
  termnamemax = 50;   { safe length for terminal name }
type
  labstrtype= string[keylabelmax];  
var
  c2: chtype; { chtype is the ncurses "char" type (actually integer) returned by getch etc }
  KeyLabels: array[KEY_MIN .. KEY_MAX] of labstrtype;
  HasKeySet: set of KEY_MIN..KEY_MAX;  
  TermNamePS: string(termnamemax);

end { of module-heading};


{ module block is below, these functions are not exported }


function KeyNameClean(c: chtype): labstrtype;
  { return a more user-oriented key name, e.g. "KEY_F(2)"  => "F2" , "KEY_SUNDO" => "Shift-UNDO". }
const safelen = 2 * keylabelmax;
var sx: string[safelen]; 
    ix: integer;
begin
   { ASSERT: c is a function key }
         begin { function key }
           sx := CString2String(keyname(c));
           ix :=  index(sx,'KEY_');
           if (ix=1) and (length(sx)>4) then sx :=  substr(sx,5);
           ix := index(sx,'(');
           if (ix>1) and (ix+1<length(sx)) then 
             begin  
               sx := sx[1..ix-1] + sx[ix+1..length(sx)];  { trim the "(" }
               if sx[length(sx)] = ')' then sx :=substr(sx,1,length(sx)-1);
             end;
           if (length(sx)>1)
             then if (sx[1]='S') then
              if (c in ([KEY_SBEG .. KEY_SUNDO] - [KEY_SELECT])) then { Shifted function key }
                sx := Shift  + substr(sx,2,length(sx)-1) ;
        end;
  if length(sx)>keylabelmax 
      then KeyNameClean := sx[1..keylabelmax] { unlikely but safe }
      else KeyNameClean := sx;
end { KeyNameClean };


procedure CreateKeyLabelArray;
{ Fill the array of strings "KeyLabels". }
{ First, get the labels returned by the brain-damaged "keyname", via local "KeyNameClean" }
{ Then, put in any labels defined in terminfo (which only allows definitions for F0 - F10) }
var
  cparam: CString;
  keylab: string(20);
  c1: 0..KEY_MAX; { theoretically of type chtype, but this safe range avoids set limit warning }
begin
   HasKeySet := [];
   for  c1 := KEY_MIN to KEY_MAX do
     begin
       KeyLabels[c1] := KeyNameClean(c1);
       if has_key(c1) then HasKeySet := HasKeySet + [c1];
     end;
   for c1 := KEY_F0 to  KEY_F10 do
      begin
        if c1 = KEY_F10 
           then cparam := 'lf10'
           else cparam := 'lf' + chr( ord('0') + c1 - KEY_F0 );
        keylab := CString2String ( tigetstr(cparam) );  { gets the label in terminfo }
        if length(keylab) > 0 then { alternate label was defined in terminfo }
           KeyLabels[c1] := keylab; 
      end;
end { CreateKeyLabelArray };


procedure NameLabelTerminalAdjust;
{ First, fix some of the more inscrutable key labels, e.g. IC IL, DC.  }
{ Then, check for a few known terminals, e.g. vt220 key is "Do", not "REDO". }
{ Here we slice and compare based on what is in the terminfo data base as of late 2005. }
{ Note that these adjustments rely on "set term=xxx" to be set correctly!  All of }
{  ncurses relies on this, but the problem is more dicey for key names because }
{  emulators mostly get screen codes right but can't change what is physically }
{  written on keyboard (except via on-screen key layouts). }
var
  termnamevar: string(termnamemax); { local var Pascal string for terminal name }
  term5: string(5);
  term7: string(7);
  keydef: CString;
begin
   { terminal-independent fixes for inscrutable names:  DC, IC, DL, IL SDC, SIC, SDL }
   begin
    KeyLabels[KEY_DC]:='DelCh';  KeyLabels[KEY_SDC]:= Shift + 'DelCh';
    KeyLabels[KEY_IC]:='InsCh';  KeyLabels[KEY_SIC]:= Shift + 'InsCh';  
    KeyLabels[KEY_DL]:='DelLn';  KeyLabels[KEY_SDL]:= Shift + 'DelLn';
    KeyLabels[KEY_IL]:='InsLn';  
    { KeyLabels[KEY_SIL]:= Shift + 'InsLn'; }  { oddly, SIL not in ncurses}
    { substitute for RIGHT and LEFT is somewhat risky--e.g. may look bad with oversized ">" char }
    KeyLabels[KEY_RIGHT]:='->';  KeyLabels[KEY_SRIGHT] := Shift + ' ->';
    KeyLabels[KEY_LEFT ]:='<-';  KeyLabels[KEY_SLEFT] := Shift + ' <-';
    KeyLabels[KEY_NPAGE]:='NxtPage';
    KeyLabels[KEY_PPAGE]:='PrvPage';
   end;
   termnamevar := CString2String(termname);  { termname is an ncurses function, returning type CString }
   TermNamePS := termnamevar;  { exported as pascal string }
   if length(termnamevar)<7 then termnamevar:=termnamevar+'       '; { prevent r.t. error on next 2 string slices }
   term5 := termnamevar[1..5];
   term7 := termnamevar[1..7];
   { seek true vt220s and vt320s with DEC LK201 keyboard, NOT emulators nor wyse }
   { vt220 etc cannot actually distinguish "Shift-" on these keys, but high quality emulators may }
   if (term5 = 'vt220') or (term5='vt340') or ( (term5='vt320') and (term7 <> 'vt320-k') )
      then 
        begin 
          { some termcaps do not define the key "Remove", which should be KEY_DC }
          keydef := '\E[3~';  { LK 201, key "Remove" }
          { if keybound(KEY_DC, 0) - null string,  then }
          if not has_key(KEY_DC) then 
             begin
               define_key(keydef, KEY_DC);
               if has_key(KEY_DC) then HasKeySet := HasKeySet + [KEY_DC];
             end;
          KeyLabels[KEY_DC]:='Remove'; KeyLabels[KEY_SDC]:= Shift + 'Remove'; { not defined in terminfo ;< }
          KeyLabels[KEY_REDO]:='Do'; KeyLabels[KEY_SREDO]:= Shift + 'Do';
          KeyLabels[KEY_IC]:='InsHere'; KeyLabels[KEY_SIC]:= Shift + 'InsHere';
        end;
end { NameLabelTerminalAdjust };

{----------------------- following mouse management not yet implemented ------------------- }
      { Pascal ordinal and set types, to make "mask" operations easier using Pascal sets }
      {  note: c "mask" constants have underscore, Pascal ordinal types do not }
   { NOTE, September 2005: NOT YET DEBUGGED, DO NOT USE, ALSO FUNCTION mousemaskS COMMENTED OUT }
{   type
    ButtonStateEnum = (
        Button1Released, Button1Pressed, Button1Clicked, 
        Button1DoubleClicked, Button1TripleClicked, Button1ReservedEvent,
        Button2Released, Button2Pressed, Button2Clicked, 
        Button2DoubleClicked, Button2TripleClicked, Button2ReservedEvent,
        Button3Released, Button3Pressed, Button3Clicked, 
        Button3DoubleClicked, Button3TripleClicked, Button3ReservedEvent,
        Button4Released, Button4Pressed, Button4Clicked, 
        Button4DoubleClicked, Button4TripleClicked, Button4ReservedEvent,
        ButtonCtrl,  ButtonShift,  ButtonAlt,  ReportMousePosition);
      
      ButtonStateSet = set of ButtonStateEnum;
     pButtonStateSet = ^ButtonStateSet;
 }
 {  ASSERT:  sizeof(ButtonStateSet) = sizeof(mmask_t)   }
 {  const AllMouseEvents = [Button1Released .. ButtonAlt];  }
{   FUNCTION mousemasks( _para1 : ButtonStateSet; _para2 : pButtonStateSet): ButtonStateSet; external name 'mousemask'; }
{ mousemasks is equivalent to mousemask, but uses set type ButtonStateSet rather than integer bitmask }

{ ----------------------------- end, unimplemented mouse management ----------------------- }

to begin do
  begin { to begin do }
  { writeln('Running init code of ncursesPlus ... press RETURN '); readln;  }
   initscr;  { start up ncurses }
   keypad(stdscr, true);  { enable ncurses' recognition of F keys }

   CreateKeyLabelArray;
   NameLabelTerminalAdjust;

   endwin; { terminate ncurses }

  end { to begin do };
end { end of module block } .

